package com.xpn.xwiki.xmlrpc3;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.xmlrpc.XmlRpcException;

import com.xpn.xwiki.XWiki;
import com.xpn.xwiki.XWikiContext;
import com.xpn.xwiki.XWikiException;
import com.xpn.xwiki.render.XWikiVelocityRenderer;
import com.xpn.xwiki.web.Utils;
import com.xpn.xwiki.web.XWikiRequest;
import com.xpn.xwiki.web.XWikiResponse;
import com.xpn.xwiki.web.XWikiServletContext;
import com.xpn.xwiki.web.XWikiURLFactory;

public class XWikiUtils
{
    public static Object mock(Class someClass)
    {
        ClassLoader loader = someClass.getClassLoader();

        InvocationHandler handler = new InvocationHandler()
        {
            public Object invoke(Object proxy, Method method, Object[] args) throws Throwable
            {
                return null;
            }
        };
        Class[] interfaces = new Class[] {someClass};

        return Proxy.newProxyInstance(loader, interfaces, handler);
    }

    public static com.xpn.xwiki.api.XWiki checkTokenAndGetXWikiApiWrapper(String token,
        XWikiRequest request, XWikiResponse response, XWikiServletContext servletContext)
        throws XWikiException, XmlRpcException
    {
        XWikiContext context = getXWikiContext(request, response, servletContext);
        XWikiUtils.checkToken(token, context);
        com.xpn.xwiki.api.XWiki xwiki = new com.xpn.xwiki.api.XWiki(context.getWiki(), context);

        return xwiki;
    }

    public static XWikiContext getXWikiContext(XWikiRequest request, XWikiResponse response,
        XWikiServletContext servletContext) throws XWikiException
    {
        XWikiContext context = Utils.prepareContext("", request, response, servletContext);
        XWiki xwiki = XWiki.getXWiki(context);
        XWikiURLFactory urlf =
            xwiki.getURLFactoryService().createURLFactory(context.getMode(), context);
        context.setURLFactory(urlf);
        XWikiVelocityRenderer.prepareContext(context);

        return context;
    }

    public static Map getTokens(XWikiContext context)
    {
        Map tokens = (Map) context.getEngineContext().getAttribute("xmlrpc_tokens");
        if (tokens == null) {
            tokens = new HashMap();
            context.getEngineContext().setAttribute("xmlrpc_tokens", tokens);
        }

        return tokens;
    }

    public static XmlRpcUser getUserForToken(XWikiContext context, String token)
    {
        return (XmlRpcUser) getTokens(context).get(token);
    }

    public static void checkToken(String token, XWikiContext context) throws XmlRpcException
    {
        XmlRpcUser user = null;
        String ip = context.getRequest().getRemoteAddr();

        if (token != null) {
            if (!token.equals("")) {
                user = (XmlRpcUser) getTokens(context).get(token);
            } else {
                // anonymous access
                user = new XmlRpcUser("XWiki.XWikiGuest", ip);
            }
        }

        if ((user == null) || (!user.getRemoteIp().equals(ip))) {
            throw new XmlRpcException(String.format(
                "Access Denied: authentication token '%s' for IP %s is invalid", token, ip));
        }

        context.setUser(user.getUserName());
    }
}
