package com.xpn.xwiki.xmlrpc3;

import java.io.IOException;
import java.net.URL;

import javax.servlet.ServletConfig;
import javax.servlet.http.HttpServletRequest;

import org.apache.xmlrpc.XmlRpcException;
import org.apache.xmlrpc.XmlRpcRequest;
import org.apache.xmlrpc.common.XmlRpcHttpRequestConfigImpl;
import org.apache.xmlrpc.server.PropertyHandlerMapping;
import org.apache.xmlrpc.server.RequestProcessorFactoryFactory;
import org.apache.xmlrpc.webserver.XmlRpcServlet;
import org.apache.xmlrpc.webserver.XmlRpcServletServer;

public class XWikiXmlRpcServlet extends XmlRpcServlet
{
    private static final long serialVersionUID = 3745689092652029366L;

    @Override
    protected PropertyHandlerMapping newPropertyHandlerMapping(URL url) throws IOException,
        XmlRpcException
    {
        PropertyHandlerMapping mapping = new PropertyHandlerMapping();
        mapping.setTypeConverterFactory(getXmlRpcServletServer().getTypeConverterFactory());

        RequestProcessorFactoryFactory factory =
            new RequestProcessorFactoryFactory.RequestSpecificProcessorFactoryFactory()
            {
                protected Object getRequestProcessor(Class pClass, XmlRpcRequest pRequest)
                    throws XmlRpcException
                {
                    Object proc = super.getRequestProcessor(pClass, pRequest);

                    if (proc instanceof XWikiRpcHandler) {
                        XWikiRpcHandler handler = (XWikiRpcHandler) proc;
                        HttpServletRequest request =
                            ((XWikiXmlRpcHttpRequestConfig) pRequest.getConfig()).getRequest();

                        handler.init(XWikiXmlRpcServlet.this, request);
                    }

                    return proc;
                }
            };

        mapping.setRequestProcessorFactoryFactory(factory);
        mapping.load(Thread.currentThread().getContextClassLoader(), url);
        
        return mapping;

    }

    @Override
    protected XmlRpcServletServer newXmlRpcServer(ServletConfig config) throws XmlRpcException
    {        
        return new XmlRpcServletServer()
        {
            @Override
            protected XmlRpcHttpRequestConfigImpl newConfig(HttpServletRequest request)
            {
                return new XWikiXmlRpcHttpRequestConfig(request);
            }
        };
    }
}
